# Generated by Django 4.2.9 on 2024-04-15 22:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='OptionType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('python_callable', models.CharField(max_length=2048)),
                ('documentation', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='Option',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=2048)),
                ('value', models.CharField(max_length=2048)),
                ('default_value', models.CharField(max_length=2048)),
                ('default_value_change_behavior', models.CharField(choices=[('always', 'Always'), ('never', 'Never'), ('change_if_unchanged', 'Change If Unchanged')], max_length=2048)),
                ('documentation', models.TextField()),
                ('default_type', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='defaultoptiontype', to='deployment_configuration.optiontype')),
                ('option_type', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='deployment_configuration.optiontype')),
                ('supported_types', models.ManyToManyField(related_name='supportedoptiontypes', to='deployment_configuration.optiontype')),
            ],
        ),
    ]
