# Generated by Django 5.0.6 on 2024-08-05 18:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('deployment_configuration', '0003_alter_option_supported_types'),
    ]

    operations = [
        migrations.AlterField(
            model_name='option',
            name='default_value_change_behavior',
            field=models.CharField(choices=[('always_change', 'Always Change'), ('never_change', 'Never Change'), ('change_if_unchanged', 'Change if Unchanged')], default='always_change', max_length=2048),
        ),
    ]
