# Generated by Django 4.2.9 on 2024-04-15 23:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('deployment_configuration', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='option',
            old_name='value',
            new_name='raw_value',
        ),
    ]
